@@ -5,91 +5,8 @@
 local describe, it, assert, mock, setup, before_each =
       describe, it, assert, mock, setup, before_each  -- luacheck: ignore
 
--- gloabl varables to set $XDG_CONFIG_HOME and $XDG_DATA_HOME to for the
--- tests.
-local tmp = os.getenv("TMPDIR") or "/tmp"
-local confdir = tmp .. "/nvimpager-testsuite/no-config"
-local datadir = tmp .. "/nvimpager-testsuite/no-data"
-
--- Run a shell command, assert it terminates with return code 0 and return its
--- output.
---
--- The assertion of the return status works even with Lua 5.1.  The last byte
--- of output of the command *must not* be a decimal digit.
---
--- command: string -- the shell command to execute
--- returns: string -- the output of the command
-local function run(command)
-  -- From Lua 5.2 on we could use io.close to retrieve the return status of
-  -- the process.  It would return true, "exit", x where x is the status.
-  -- For Lua 5.1 (currently used by neovim) we have to echo the return status
-  -- in the shell command and extract it from the output.
-  -- References:
-  -- https://www.lua.org/manual/5.1/manual.html#pdf-io.close
-  -- https://www.lua.org/manual/5.1/manual.html#pdf-file:close
-  -- https://www.lua.org/manual/5.2/manual.html#pdf-io.close
-  -- https://www.lua.org/manual/5.2/manual.html#pdf-file:close
-  -- https://www.lua.org/manual/5.2/manual.html#pdf-os.execute
-  -- https://stackoverflow.com/questions/7607384
-  command = string.format("XDG_CONFIG_HOME=%s XDG_DATA_HOME=%s %s; echo $?",
-    confdir, datadir, command)
-  local proc = io.popen(command)
-  local output = proc:read('*all')
-  local status = {proc:close()}
-  -- This is *not* the return value of the command.
-  assert.equal(true, status[1])
-  -- In Lua 5.2 we could also assert this and it would be meaningful:
-  -- assert.equal("exit", status[2])
-  -- assert.equal(0, status[3])
-  -- For Lua 5.1 we have echoed the return status with the output.  First we
-  -- assert the last two bytes, which is easy:
-  assert.equal("0\n", output:sub(-2), "command failed")
-  -- When the original command did not produce any output this is it.
-  if #output ~= 2 then
-    -- Otherwise we can only hope that the command did not produce a digit as
-    -- it's last character of output.
-    assert.is_nil(tonumber(output:sub(-3, -3)), "command failed")
-  end
-  -- If the assert succeeded we can remove two bytes from the end.
-  return output:sub(1, -3)
-end
-
--- Read contents of a file and return them.
---
--- filename: string -- the name of the file to read
--- returns: string -- the contents of the file
-local function read(filename)
-  local file = io.open(filename)
-  local contents = file:read('*all')
-  return contents
-end
-
--- Freshly require the nvimpager module, optinally with mocks
---
--- api: table|nil -- a mock for the neovim api table (:help lua-api)
--- return: table -- the nvimpager module
-local function load_nvimpager(api)
-  -- Create a local mock of the vim module that is provided by neovim.
-  local default_api = {
-    nvim_get_hl_by_id = function() return {} end,
-    -- These can return different types so we just default to nil.
-    nvim_call_function = function() end,
-    nvim_get_option = function() end,
-  }
-  if api == nil then
-    api = default_api
-  else
-    for key, value in pairs(default_api) do
-      if api[key] == nil then api[key] = value end
-    end
-  end
-  local vim = { api = api }
-  -- Register the api mock in the globals.
-  _G.vim = vim
-  -- Reload the nvimpager script
-  package.loaded["lua/nvimpager"] = nil
-  return require("lua/nvimpager")
-end
+local helpers = require("test/helpers")
+local run, read = helpers.run, helpers.read
 
 describe("auto mode", function()
   -- Auto mode only exists during the run of the bash script.  At the end of
@@ -110,7 +27,7 @@ describe("auto mode", function()
       set -e
       set -u
       shopt -s expand_aliases
-      NVIM=:
+      NVIMPAGER_NVIM=:
       alias exec=:
       alias trap=:
       source ]] .. command .. "\nset"
@@ -159,6 +76,13 @@ describe("cat mode", function()
     assert.equal(expected, output)
   end)
 
+  it("handles color schemes with a non trivial Normal group", function()
+    local output = run("./nvimpager -c test/fixtures/conceal.tex " ..
+		       "--cmd 'hi Normal ctermfg=Red'")
+    local expected = read("test/fixtures/conceal.tex.red")
+    assert.equal(expected, output)
+  end)
+
   it("highlights all files", function()
     local output = run("./nvimpager -c test/fixtures/makefile " ..
                                       "test/fixtures/help.txt ")
@@ -202,6 +126,23 @@ describe("cat mode", function()
     assert.equal(read("test/fixtures/makefile.ansi"), output)
   end)
 
+  describe("can change the default foreground color", function()
+    for termguicolors, extension in pairs({termguicolors = "red24", notermguicolors = "red"}) do
+      for _, command in pairs({"--cmd", "-c"}) do
+	for stdin, redirect in pairs({[false] = "", [true] = "<"}) do
+	  it("with "..command..", setting "..termguicolors..(stdin and " input via stdin" or ""), function()
+	    local script = "./nvimpager -c -- " .. command ..
+	      " 'highlight Normal ctermfg=red guifg=red | set " ..
+	      termguicolors .. "' " .. redirect .. "test/fixtures/plain.txt"
+	    local output = run(script)
+	    local expected = read("test/fixtures/plain." .. extension)
+	    assert.equal(expected, output)
+	  end)
+	end
+      end
+    end
+  end)
+
   describe("with modeline", function()
     it("highlights files even after mode line files", function()
       local output = run("./nvimpager -c test/fixtures/conceal.tex " ..
@@ -339,14 +280,14 @@ describe("backend:", function()
     -- Nvim writes this message to stderr so we have to redirect this.
     local output = run("./nvimpager -c -- README.md " ..
                        "-c 'echo $NVIM_RPLUGIN_MANIFEST' -c quit 2>&1")
-    assert.equal(datadir..'/nvimpager/rplugin.vim', output)
+    assert.equal(helpers.datadir..'/nvimpager/rplugin.vim', output)
   end)
 end)
 
 describe("lua functions", function()
   local nvimpager
 
-  setup(function() nvimpager = load_nvimpager() end)
+  setup(function() nvimpager = helpers.load_nvimpager() end)
 
   describe("split_rgb_number", function()
     it("handles numbers from 0 to 16777215", function()
@@ -377,7 +318,7 @@ describe("lua functions", function()
 	  nvim_call_function = function() return 0 end,
 	}
 	local m = mock(api)
-	local nvimpager = load_nvimpager(api)
+	local nvimpager = helpers.load_nvimpager(api)
 	nvimpager._testable.init_cat_mode()
 	local escape = nvimpager._testable.group2ansi(100)
 	assert.stub(m.nvim_get_hl_by_id).was.called_with(100, termguicolors)
@@ -738,6 +679,13 @@ describe("parent detection", function()
     assert.equal(expected, output)
   end)
 
+  it("can pass though git colors", function()
+    local output = run("test/fixtures/bin/git ./nvimpager -c " ..
+		       "test/fixtures/difftastic --cmd 'lua nvimpager.git_colors=true'")
+    local expected = read("test/fixtures/difftastic")
+    assert.equal(expected, output)
+  end)
+
   it("handles man #osx_pending", function()
     local output = run("test/fixtures/bin/man ./nvimpager -c " ..
 		       "test/fixtures/man.cat")
@@ -745,3 +693,52 @@ describe("parent detection", function()
     assert.equal(expected, output)
   end)
 end)
+
+describe("init files", function()
+  it("can be specified with -u", function()
+    local init = os.tmpname()
+    finally(function() os.remove(init) end)
+    helpers.write(init, "let g:myvar = 42")
+    local output = run("./nvimpager -c -- -u " .. init ..
+      [[ -c 'lua io.write(vim.g.myvar, "\n")' -c qa]])
+    assert.equal("42\n", output)
+  end)
+
+  local function tempdir()
+    local dir = run("mktemp -d"):sub(1, -2)  -- remove the final newline
+    finally(function() run("rm -r " .. dir) end)
+    return dir
+  end
+
+  it("can be init.lua", function()
+    local dir = tempdir()
+    run("mkdir -p " .. dir .. "/nvimpager")
+    helpers.write(dir .. "/nvimpager/init.lua", "vim.g.myvar = 42")
+    local output = run("XDG_CONFIG_HOME=" .. dir ..
+      [[ ./nvimpager -c -- -c 'lua io.write(vim.g.myvar, "\n")' -c qa]])
+    assert.equal("42\n", output)
+  end)
+
+  it("can be init.vim", function()
+    local dir = tempdir()
+    run("mkdir -p " .. dir .. "/nvimpager")
+    helpers.write(dir .. "/nvimpager/init.vim", "let myvar = 42")
+    local output = run("XDG_CONFIG_HOME=" .. dir ..
+      [[ ./nvimpager -c -- -c 'lua io.write(vim.g.myvar, "\n")' -c qa]])
+    assert.equal("42\n", output)
+  end)
+
+  it("can not coexist (.lua and .vim)", function()
+    local dir = tempdir()
+    local conf = dir .. "/nvimpager"
+    run("mkdir -p " .. conf)
+    helpers.write(conf .. "/init.lua", "vim.g.myvar = 41")
+    helpers.write(conf .. "/init.vim", "let myvar = 43")
+    local output = run("XDG_CONFIG_HOME=" .. dir ..
+      [[ ./nvimpager -c -- -c 'lua io.write(vim.g.myvar, "\n")' -c qa;
+      test $? -eq 2]])
+    local expected = "Conflicting configs: " .. conf .. "/init.lua " ..
+      conf .. "/init.vim\n"
+    assert.equal(expected, output)
+  end)
+end)
